/*
** eco.c
**
** Made by Patrick MARIE (linfg122)
** <mycroft@minithins.net> on marine.edu.ups-tlse.fr
**
** Started on  Thu Nov 21 15:09:50 2002 linfg122
** Last update dim 24 nov 2002 18:22:50 CET mycroft
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include "listes.h"
#include "fonctions.h"
#include "system.h"
#include "vie.h"

int generate_gfx (struct monde_t *le_monde, cel *entites, pcel liste_objets)
{
  int i;
  int j;
  cel *tmp;

  /* on remplit de barre. */

  memset (le_monde->tabgfx, '_', (le_monde->taille_x + 2) * (le_monde->taille_y + 2));


  /* on doit recopier le tabland dans le tab gfx */

  for (i = 0; i < le_monde->taille_y; i ++)
  {
    for (j = 0; j < le_monde->taille_x; j ++)
    {
      le_monde->tabgfx[((2 + le_monde->taille_x) * (i + 1) + (j + 1))] 
        = le_monde->tabland[(i * le_monde->taille_x) + j]; 
    }
  }

  le_monde->tabgfx[0] = '+';

  for (i = 1; i < le_monde->taille_x + 1; i ++)
    le_monde->tabgfx[i] = '-';

  le_monde->tabgfx[le_monde->taille_x + 1] = '+';

  for (i = 1; i < le_monde->taille_y + 1; i ++)
  {
    le_monde->tabgfx[i * (le_monde->taille_x + 2)] = '|';
    le_monde->tabgfx[i * (le_monde->taille_x + 2) + le_monde->taille_x + 1] = '|';
  }

  for (i = 1; i < le_monde->taille_x + 1; i ++)
    le_monde->tabgfx[i + (le_monde->taille_y + 1) * (le_monde->taille_x + 2)] = '-';

  le_monde->tabgfx[(le_monde->taille_x + 2) * (le_monde->taille_y + 1)] = '+';
  le_monde->tabgfx[(le_monde->taille_x + 2) * (le_monde->taille_y + 2) - 1] = '+';

  tmp = entites;

  while (tmp)
  {
    /* On dessine et on met les char des especes dans la matrice */
    le_monde->tabgfx[((struct entite_t*)CEL_CONT(tmp))->y * (le_monde->taille_x + 2) + ((struct entite_t *)CEL_CONT(tmp))->x]
      = get_char_espece (((struct entite_t*)CEL_CONT(tmp))->espece, liste_objets);
    tmp = CEL_NEXT (tmp);
  }

  return (0);
}

int affiche_gfx (struct monde_t *le_monde)
{

  int i, j;

  for (i = 0; i < le_monde->taille_y + 2; i++)
  {
    for (j = 0; j < le_monde->taille_x + 2; j++)
      fprintf (stdout, "%c", le_monde->tabgfx[(i * (le_monde->taille_x + 2)) + j]);

    fprintf (stdout, "\n");
  }

  return (0);
}

/*
** La fonction eco systeme fait:
**
** -> sauvegarde automatique
** -> afficher le monde (generer monde->tabfxg)
** -> attendre pour un ordre
** -> calculer le nouvel etat
** -> boucler
**
*/

int eco_systeme (struct monde_t **monde, cel **entites, pcel liste_objets)
{
  int fin = 0;
  int temps_passe = 0;
  int temps_present = 0; /* warnings qui ne devraient pas y etre . */
  int temps_futur = 0;
  int c;
  cel *tmp;
  cel *stmp;
  int ret;

  /* Nombre de cycles a faire avant de redemander des ordres */
  int cycle_auto = 1;


  if (!*monde || !*entites)
  {
    fprintf(stderr, "Le jeu ne peut fonctionner: Vous devez charger une partie.\n");
    set_return_code (E_NAPASPLUSDEMARRER_LE_JEU);
    return (-1);
  }

  do
  {
    /* afficher le monde */

    if (cycle_auto == 1)
    {
      generate_gfx (*monde, *entites, liste_objets);
      affiche_gfx (*monde);
    }

    /* attendre pour un ordre */
    if (!(--cycle_auto))
    {
      fprintf(stdout, "Cycle n%i\n", (*monde)->cycle);
      if (!*entites)
      { 
        fprintf(stdout, "Il n'y a plus d'entites dans le monde. ;-(\n");
        set_return_code (E_YAPLUSDEBESTIOLLE);
      }
      /* fait rien encore */
      c = 0;
      do
      {
        if (c)
          set_return_code (E_BAD_USER);
        fprintf (stdout, "\n\nMenu:\n");
        fprintf (stdout, "n/ Etape suivante\n");
        fprintf (stdout, "b/ Etape precedente\n");
        fprintf (stdout, "r/ Passer les 10 prochaines cycles\n");
        fprintf (stdout, "u/ Passer les N prochains cycles\n");
        fprintf (stdout, "s/ Sauvegarde\n");
        fprintf (stdout, "f/ Retour au menu\n");
        fprintf (stdout, "\nVotre choix: ");
        fflush (stdout);
 
        c = my_getchar();
      }
      while (c != 'r' && c != 'b' && c != 'n' 
          && c != 'f' && c != 'u' && c != 's' && c != '\n' && c != ' ');

      /* n = next, suivant
      ** b = back, precedent
      ** r = run
      ** f = final 
      */

      switch (c)
      {
        default:
          set_return_code (E_BAD_CODER);
        break;
        case 'r':
          cycle_auto = 10;
        break;

        case 'u':
          do
          {
            fprintf(stdout, "Nombre de cycles  passer: "); fflush (stdout);
            ret = fscanf (stdin, "%i", &cycle_auto);
            if (ret == 1) 
              vide_buffer ();
          }
          while (ret != 1);
        break;
        case 'b':
          if ((*monde)->past > 0)
          {
            fprintf(stdout, "Retour en %i\n", (*monde)->past);
            charger_jeu (0, monde, entites, (*monde)->past);
            cycle_auto = 1;
            continue ;
          }
          else
          {
            set_return_code (E_BAD_USER);
            fprintf(stdout, "Nous sommes au debut du monde, on ne peut pas faire mieux.\n");
            cycle_auto = 1;
            continue ;
          }
        break;

		case '\n':
		case ' ':
        case 'n':
          cycle_auto = 1;
          if ((*monde)->future)
          {
            fprintf(stdout, "Swapping to another world ...\n");
            if (!charger_jeu (0, monde, entites, (*monde)->future))
              continue ;
            else
              (*monde)->future = 0;
          }
        break;
        case 's':
          sauvegarder_jeu (*monde, *entites, 0);
          cycle_auto = 1;
          continue ; 
        break;
        case 'f':
          return (0);
        break;
      }

      fprintf(stdout, "\n\n\n\n\n\n");
    }

    if (!(*monde)->future)
    {
      /* sauvegarde automatique */
      temps_passe   = (*monde)->present; 
      temps_present = temps_present ++;
      temps_futur   = 0; /* pas connu encore ... */

      (*monde)->future = temps_present;

      /*
      ** Ici on sauvegarde ...
      */

      sauvegarder_jeu (*monde, *entites, temps_passe); 
      /* le dernier n'a aucune valeur, faut juste que ca soit  != 0 */

      /* calculer le nouvel etat */

      tmp = *entites;

      while (tmp)
      {
        ((struct entite_t *)CEL_CONT(tmp))->vie --;

        tmp = CEL_NEXT(tmp);
      }

      tmp = *entites;

      while (tmp)
      {
        /*
         Ok on a  retirer une cellule.
         */
        if (((struct entite_t *)CEL_CONT(tmp))->vie <= 0)
        {
          stmp = CEL_NEXT (tmp);
          virer_liste (entites, tmp);
          tmp = stmp;
        }
        else
          tmp = CEL_NEXT(tmp);
      }

      (*monde)->past    = (*monde)->present;
      (*monde)->present = temps_present;
      (*monde)->future  = 0;
      (*monde)->cycle ++;

      /* Vie future ...*/
      /* On calcule ici le nouveau cycle */

      new_world (monde, entites, liste_objets);
    }
  }
  while (!fin);

  return (0);
}
