/*
** entite.c
**
** Made by Patrick MARIE (linfg122)
** <mycroft@minithins.net> on marine.edu.ups-tlse.fr
**
** Started on  Fri Nov 08 10:59:37 2002 linfg122
** Last update dim 24 nov 2002 10:40:20 CET mycroft
*/

/*
 * Fonctions relatives aux entites.
 */
#include <stdlib.h>
#include <stdio.h>

#include "fonctions.h"
#include "listes.h"
#include "entite.h"
#include "monde.h"

cel *charger_objets (const char *filename)
{
  cel *head = creer_liste ();
  struct objet_t *new = NULL;
  FILE *hand;
  int ret;

  hand = fopen (filename, "r");
  if (!hand)
  {
    set_return_code (E_INPUT_OUTPUT);
    fprintf (stderr, "Ne peut pas ouvrir le fichier %s\n",
                     filename);

    vider_liste (&head);
    return (NULL);
  }

  do
  {
    new = my_malloc (sizeof (struct objet_t));
    if (!new)
    {
      fprintf(stderr, "Ne peut allouer de la memoire.");
      fclose (hand);
      return (head);
    }

    ret = fscanf (hand, "%d %s %c %d %d %d %d %d %d %d %d %i %d %d %d %d %d %d %d %d %d %d %d %d",
      &(new->espece), (new->nom_espece), &(new->carac),
      &(new->jemange.ch_esp), &(new->tumemanges.ch_esp),
      &(new->mvt_plaine), &(new->mvt_montagne),
      &(new->mvt_eau), &(new->mvt_neige), &(new->mvt_foret),
      &(new->mvt_desert), &(new->pt_repro),  &(new->max_pt_eat),
      &(new->max_pt_mov), &(new->max_pt_rep), &(new->max_pt_ter),
      &(new->max_pt_med), &(new->max_pt_arm), &(new->max_pt_mut),
      &(new->max_pt_sci), &(new->max_pt_bra), &(new->max_pt_utf),
      &(new->max_pt_pmp), &(new->max_pt_42) 
      );

    if (ret == 24)
      ajout_liste (&head, new);
    else
    { 
      if (ret != -1)
      set_return_code ( E_FICHIER_OBJET );
      my_free (new);
    } 
  }
  while (ret == 24);

  fclose (hand);
  return (head);
}

int retourne_pt_repro_par_espece (cel *liste, unsigned char espece)
{
  int a = 1000;
  pcel lala;
 
  lala = retourne_objet_par_espece (liste, espece);
  if (lala)
    a = ((struct objet_t *)CEL_CONT(lala))->pt_repro;

  return (a);
}

cel *retourne_objet_par_carac (cel *liste, unsigned char espece)
{
  cel *tmp = liste;

  while (tmp)
  {
    if (((struct objet_t*)(tmp->cel_info))->carac == espece)
      return (tmp);

    tmp = tmp->cel_suiv;
  }
  
  return (NULL);
} 

cel *retourne_objet_par_espece (cel *liste, unsigned int espece)
{
  cel *tmp = liste;

  while (tmp)
  {
    if (((struct objet_t*)(tmp->cel_info))->espece == espece)
      return (tmp);

    tmp = tmp->cel_suiv;
  }

  return (NULL);
}

int print_objets (cel *head)
{
  cel *tmp = head;
  int i = 0;

  fprintf (stdout, "Liste des especes connues:\n");
  while (tmp)
  {
    fprintf (stdout, "%c > %s\n",
      ((struct objet_t*)(tmp->cel_info))->carac,
      ((struct objet_t*)(tmp->cel_info))->nom_espece);
    tmp = tmp->cel_suiv;
    i ++ ;
  }

  fprintf (stdout, "\n");
  return (i);
}

void libere_objets (cel *head)
{
  vider_liste (&head);
  return ;
}

cel *charger_entites (const char *filename)
{
  FILE *hand;
  pcel head = NULL;
  int ret;
  struct entite_t *el;
 
  hand = fopen (filename, "r");

  do
  {
    el = my_malloc (sizeof (struct entite_t));
    if (!el)
    {
      fprintf(stderr, "Erreur memoire pendant le chargement des entites\n");
      fclose (hand);
      return (head);
    }

    ret = fscanf(hand,"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n",
                  &(el->espece), &(el->x), &(el->y), &(el->vie),
                  &(el->pt_eat), &(el->pt_mov), &(el->pt_rep),
                  &(el->pt_ter), &(el->pt_med), &(el->pt_arm),
                  &(el->pt_mut), &(el->pt_sci), &(el->pt_bra),
                  &(el->pt_utf), &(el->pt_pmp), &(el->pt_42)
                 );
    if (ret == 16)
      ajout_liste (&head, el);
    else
    {
      if (ret != -1)
        set_return_code (E_FICHIER_ENTITE);
      my_free (el);
    }
  }
  while (ret == 16);

  fclose (hand);
  return (head);
}

cel *creer_entites (int x, int y, cel *obj)
{
  /* x: taille largeur, y: taille longueur */

  cel *head = NULL;
  cel *l_objet = NULL;
  struct entite_t *el;
  char c;
  int num ;
  char n;

  do
  {
    /* Creation de la cellule */
    el = my_malloc (sizeof (struct entite_t));

    fprintf (stdout, "\n\nAjout d'une entite vivante dans le monde:\n");

    /* Saisie du type de l objet.Saisie du type de l objet.  */

    num = print_objets (obj);

    do
    {
      fprintf (stdout, "Espece choisie: ");
      fflush (stdout);

      if (fscanf (stdin, "%c", &n) != 1)
      {
        vide_buffer ();
        fprintf (stderr, "Saisie incorrecte ...\n");
        set_return_code (E_BAD_USER);
      }
      else
      {
        if (!(l_objet = retourne_objet_par_carac (obj, n)))
        {
          fprintf (stderr, "Cet objet n'existe pas ...\n");
          set_return_code (E_BAD_USER);
        }
      }
    }
    while (!l_objet);

    /* Coordonnees de l'objet */

    do
    {
      fprintf(stdout, "Coordonnee en largeur (horizontale): ");
      fflush (stdout);

      if (fscanf (stdin, "%d", &el->x) != 1)
      {
        vide_buffer ();
        fprintf (stderr, "Erreur: Le nombre entr est incorrect.\n");
        set_return_code (E_BAD_USER);
        el->x = -1;
      }
      else
        if (el->x <= 0 || el->x >= MAX_WORLD_SIZE)
        { 
          fprintf (stderr, "Erreur: Le nombre entr est incorrect.\n");
          set_return_code (E_BAD_USER);
        }
    }
    while (el->x <= 0 || el->x >= MAX_WORLD_SIZE);

    do
    {
      fprintf(stdout, "Coordonnee en longueur (verticale): ");
      fflush (stdout);

      if (fscanf (stdin, "%d", &el->y) != 1)
      {
        vide_buffer ();
        fprintf (stderr, "Erreur: Le nombre entr est incorrect.\n");
        set_return_code (E_BAD_USER);
        el->y = -1;
      }
      else
        if (el->y <= 0 || el->y >= MAX_WORLD_SIZE)
        {
          fprintf (stderr, "Erreur: Le nombre entr est incorrect.\n");
          set_return_code (E_BAD_USER);
        }
    }
    while (el->y <= 0 || el->y >= MAX_WORLD_SIZE);

    vide_buffer ();

    do
    {
      fprintf(stdout, "Vitalite de la bestiolle: ");
      fflush (stdout);

      if (fscanf (stdin, "%d", &el->vie) != 1)
      {
        vide_buffer ();
        fprintf (stderr, "Erreur: Le nombre entr est incorrect.\n");
        set_return_code (E_BAD_USER);
        el->x = -1;
      } 
      else
        if (el->x <= 0 || el->x >= DEFAULT_LIFE)
        {
          fprintf (stderr, "Erreur: Le nombre entr est incorrect.\n");
          set_return_code (E_BAD_USER);
        }
          
    }
    while (el->x <= 0 || el->x >= DEFAULT_LIFE);

    vide_buffer ();

    el->pt_eat = DEFAULT_PT_EAT;
    el->pt_mov = DEFAULT_PT_MOV;
    el->pt_rep = DEFAULT_PT_REP;

    el->espece = ((struct objet_t *)CEL_CONT(l_objet))->espece;

    ajout_liste (&head, el);

    do
    {
      fprintf (stdout, "Continuer (o/n) ? ");
      fflush (stdout);

      c = my_getchar ();
    }
    while (c != 'o' && c != 'n');
  }
  while (c == 'o');

  return (head);
}

void libere_entites (cel *head)
{
  vider_liste (&head);
  return ;
}

char get_char_espece (int esp, pcel liste_objets)
{
  pcel tmp = liste_objets;

  while (tmp)
  {
    if (((struct objet_t*)CEL_CONT(tmp))->espece == esp)
      return (((struct objet_t*)CEL_CONT(tmp))->carac);
    tmp = CEL_NEXT (tmp);
  }

  return ('?');
}
