/*
** jeu.c
**
** Made by Patrick MARIE (linfg122)
** <mycroft@minithins.net> on marine.edu.ups-tlse.fr
**
** Started on  Fri Nov 08 10:53:35 2002 linfg122
** Last update sam 23 nov 2002 21:52:40 CET mycroft
*/

/*
 * Ce fichier va contenir les differentes fonctions relatives au
 * jeu: grande boucle, evolution de la matrice et des etres vivants.
 */
#include <stdio.h>
#include <stdlib.h>

#include "fonctions.h" 
#include "listes.h"
#include "system.h"
#include "monde.h"
#include "entite.h"
#include "eco.h"

char menu (void)
{
  char reponse = 0;
  do
  {
    if (reponse)
      set_return_code (E_BAD_USER);

    printf ("\nMeuhnu:\n"
            " 1/ Creation d'un ecosysteme\n"
            " 2/ Charger la configuration par defaut\n"
            " 3/ Charger une autre configuration ...\n" 
            " 4/ Lancer l'ecosysteme\n"
            /* " 5/ Lancer l'ecosysteme en mode interpretation de commandes\n" */
            " 6/ Sauvegarder l'ecosysteme actuel\n"
            "\n"
            " 0/ Quitter\n"
            "Votre choix: "
            );
    fflush (stdout);
    reponse = my_getchar ();
  } while (reponse < '0' || reponse >= '7');

  return (reponse - '0');
}

int jeu (void)
{
  int reponse ;

/* Principales structures */
  struct monde_t *act_monde = NULL;
  pcel act_entites = NULL;
  pcel liste_objets;

  liste_objets = charger_objets ("files/objets.dat");
  if (!liste_objets)
    set_return_code (E_FICHIER_OBJETS_NON_CHARGE);

  do
  {
    switch ((reponse = menu ()))
    {
      case 1:
        if (act_monde != NULL)
        {
          libere_monde (act_monde);
          act_monde = NULL;
        }

        if (act_entites != NULL)
        {
          libere_entites (act_entites);
          act_entites = NULL;
        }

        creer_jeu (&act_monde, &act_entites, liste_objets);
		sauvegarder_jeu (act_monde, act_entites, 0);
      break;

      case 2:
        if (act_monde != NULL)
        {
          libere_monde (act_monde);
          act_monde = NULL;
        }

        if (act_entites != NULL)
        {
          libere_entites (act_entites);
          act_entites = NULL;
        }

        charger_jeu (0, &act_monde, &act_entites, 0);
      break;

      case 3:
        if (act_monde != NULL)
        {
          libere_monde (act_monde);
          act_monde = NULL;
        }

        if (act_entites != NULL)
        {
          libere_entites (act_entites);
          act_entites = NULL;
        }

        charger_jeu (1, &act_monde, &act_entites, 0);
      break;

      case 4:
        if (act_monde == NULL || act_entites == NULL)
        {
          fprintf(stderr, "Ne peut pas commencer un jeu sans avoir initialis le monde.\n");
          set_return_code (E_NAPASPLUSDEMARRER_LE_JEU);
        }
        else
        {
          eco_systeme (&act_monde, &act_entites, liste_objets);
        }
      break;

      case 6:
        sauvegarder_jeu (act_monde, act_entites, 0);
      break ;

      default:
        fprintf(stderr, "YA PAS (encore le truc defini dans le switch)\n");
        set_return_code (E_BAD_CODER);
      break;

      case 0:
        fprintf(stdout, "Fin demandee...\n");
      break;
    }
  } while (reponse != 0);

  libere_monde (act_monde);
  libere_entites (act_entites);
  libere_objets (liste_objets);

  return (0);
}
