/*
** monde.c
**
** Made by Patrick MARIE (linfg122)
** <mycroft@minithins.net> on marine.edu.ups-tlse.fr
**
** Started on  Fri Nov 08 10:56:28 2002 linfg122
** Last update sam 23 nov 2002 00:04:49 CET mycroft
*/

/*
 * Evolution du monde ( Hors entites )
 * Creation de la matrice, remplissage, etc.
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "fonctions.h"
#include "monde.h"

int verify_monde (unsigned char ptr[], int l1, int l2)
{
  unsigned char *tmp = ptr;
  int i;

  for (i = 0; i < (l1 * l2); i ++)
  {

    /* diffrents terrains */
    if (
          *tmp != '.'
       && *tmp != '^'
       && *tmp != '~'
       && *tmp != '_'
       && *tmp != '*'
       && *tmp != '#'
       )
    {
      set_return_code (E_TERRAIN_MAL_FORME);
      return (1);
    }

    tmp ++;
  }

    
  return (0);
}

struct monde_t *charger_monde (const char *fichier_monde)
{
  FILE *hand;
  pmonde le_monde;
  int ret;

  hand = fopen (fichier_monde, "r");
  if (!hand)
  {
    fprintf (stderr, "Le fichier monde n a pas pu etre ouvert.\n");
    fprintf (stderr, "Verifiez que le fichier %s existe et"
                     " qu'il n'y est pas de probleme de permissions.\n",
                     fichier_monde);
    set_return_code(E_INPUT_OUTPUT);
    return (NULL);
  }

  le_monde = my_malloc (sizeof (struct monde_t));
  if (!le_monde)
  {
    fprintf (stderr, "Le monde ne peut etre cree: Erreur de memoire.\n");
    fclose (hand);
    return (NULL);
  }

  ret = fscanf (hand, "%d %d %d %d %d %d\n", &(le_monde->taille_x), &(le_monde->taille_y), &(le_monde->cycle),
          &(le_monde->past), &(le_monde->present), &(le_monde->future));
  if (ret != 6)
  {
    fprintf (stderr, "Le fichier contenant le monde est dans un format " 
                     "non correct.\nChargement annule.\n");
    set_return_code(E_FICHIER_MONDE);
    my_free (le_monde);
    fclose (hand);
    return (NULL);
  }

  if (!(le_monde->taille_x > 0 && le_monde->taille_x < TAB_MAX 
        && le_monde->taille_y > 0 && le_monde->taille_y < TAB_MAX))
  {
    my_free (le_monde);
    fprintf (stderr, "Le monde ne peut etre cree: Tailles invalides.\n");
    set_return_code(E_FICHIER_MONDE);
    fclose (hand);
    return (NULL);
  }

  le_monde->tabland = my_malloc (1 + (sizeof (char) 
                                 * le_monde->taille_x * le_monde->taille_y));
  if (!(le_monde->tabland))
  {
    my_free (le_monde);
    fprintf (stderr, "Le monde ne peut etre cree: Erreur de memoire.\n");
    set_return_code(E_FICHIER_MONDE);
    fclose (hand);
    return (NULL);
  }

  memset (le_monde->tabland, 0, 1 + (sizeof (char) *
                             le_monde->taille_x * le_monde->taille_y));

  if ((fread (le_monde->tabland, sizeof(char), 
              le_monde->taille_x * le_monde->taille_y, hand)
       != le_monde->taille_x * le_monde->taille_y * sizeof (char)))
  {
    my_free (le_monde->tabland);
    my_free (le_monde);
    fprintf (stderr, "Le monde ne peut etre cree: "
                     "Format du fichier non correct.\n");
    set_return_code(E_FICHIER_MONDE);
    fclose (hand);
    return (NULL);
  }

  le_monde->tabgfx = my_malloc (sizeof (char) *
                     (le_monde->taille_x + 2) *
                     (le_monde->taille_y + 2));

  if (!(le_monde->tabgfx))
  {
    my_free (le_monde->tabland);
    my_free (le_monde);
    fprintf (stderr, "Le monde ne peut etre cree: "
                     "Probleme de mmoire.\n");
    fclose (hand);
    return (NULL);
  }

  if (verify_monde(le_monde->tabland, le_monde->taille_x, le_monde->taille_y))
  {
    my_free (le_monde->tabland);
    my_free (le_monde);
    fprintf (stderr, "Le monde ne peut etre cree: "
                     "Contient des caractres non reconnus.\n");
    fclose (hand);
    return (NULL);
  }

  fclose (hand);

  return (le_monde);
}

struct monde_t *creer_monde (void)
{
  struct monde_t *le_monde;

  /* Un monde, ca se construit */
  fprintf(stdout, "Cration d'un nouveau monde:\n\n");
  le_monde = my_malloc (sizeof (struct monde_t));

  if (!le_monde)
  {
    fprintf (stderr, "Le monde ne peut etre cree: "
                     "Probleme de memoire.\n");
    return (NULL);
  }

  /* Un monde, ca se dimensionne */
  do
  {
    fprintf(stdout, "Taille en largeur: ");
    fflush (stdout);

    if (fscanf (stdin, "%d", &le_monde->taille_x) != 1)
    {
      vide_buffer ();
      fprintf (stderr, "Erreur: Le nombre entr est incorrect.\n");
      set_return_code (E_BAD_USER);
    }
    else
      if (le_monde->taille_x <= 0 || le_monde->taille_x >= MAX_WORLD_SIZE)
      {
        set_return_code (E_BAD_USER);
        fprintf (stderr, "Erreur: Le nombre entr est incorrect.\n");
      }
    
  }
  while (le_monde->taille_x <= 0 || le_monde->taille_x >= MAX_WORLD_SIZE);

  do
  {
    fprintf(stdout, "Taille en longueur: ");
    fflush (stdout);

    if (fscanf (stdin, "%d", &le_monde->taille_y) != 1)
    {
      vide_buffer ();
      fprintf (stderr, "Erreur: Le nombre entr est incorrect.\n");
      set_return_code (E_BAD_USER);
    }
    else
      if (le_monde->taille_y <= 0 || le_monde->taille_y >= MAX_WORLD_SIZE)
      {
        fprintf (stderr, "Erreur: Le nombre entr est incorrect.\n");
        set_return_code (E_BAD_USER);
      }
  }
  while (le_monde->taille_y <= 0 || le_monde->taille_y >= MAX_WORLD_SIZE);

  vide_buffer ();

  le_monde->tabland = my_malloc (1 + (sizeof (char)
                                 * le_monde->taille_x * le_monde->taille_y));
  if (!(le_monde->tabland))
  {
    my_free (le_monde);
    fprintf (stderr, "Le monde ne peut etre cree: Erreur de memoire.\n");
    return (NULL);
  }

  memset (le_monde->tabland, 0, 1 + (sizeof (char) *
                             le_monde->taille_x * le_monde->taille_y));
  memset (le_monde->tabland, '.', le_monde->taille_x * le_monde->taille_y);

  le_monde->tabgfx = my_malloc (sizeof (char) *
                     (le_monde->taille_x + 2) *
                     (le_monde->taille_y + 2));

  if (!(le_monde->tabgfx))
  {
    my_free (le_monde->tabland);
    my_free (le_monde);
    fprintf (stderr, "Le monde ne peut etre cree: "
                     "Probleme de memoire.\n");
    return (NULL);
  }

  le_monde->cycle = 0;

  /* Past, present ... and future ? */
  le_monde->past = 0;
  le_monde->present = (int) time (NULL);
  le_monde->future = 0;

  return (le_monde);
}

int libere_monde (struct monde_t *monde)
{
  if (!monde)
    return (1);

  if (monde->tabland)
    my_free (monde->tabland);

  if (monde->tabgfx)
    my_free (monde->tabgfx);

  my_free (monde);

  return (0);
}
