/*
** system.c
**
** Made by Patrick MARIE (linfg122)
** <mycroft@minithins.net> on marine.edu.ups-tlse.fr
**
** Started on  Fri Nov 08 10:54:51 2002 linfg122
** Last update sam 23 nov 2002 00:21:38 CET mycroft
*/

/*
 * Fonctions de chargement, sauvegarde, etc des envs, matrices, etc.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <limits.h>
#include <stdio.h>

#include "fonctions.h"
#include "listes.h"
#include "system.h"
#include "monde.h"
#include "entite.h"

int check_rep (void)
{
  struct stat temp;

  if (stat(SAUVE_REP, &temp) != 0)
    return (-1);
  else
    return (0);
}

/*
 * Charger un jeu.
 *
 * renvoit 0 si c'est OK
 * renvoit -1 au cas d'une erreur
 *
 * Parametre: 0 == configuration par defaut.
 *            1 == charger une conf avec un nom.
 */
int charger_jeu (int mode, struct monde_t **monde, cel **entites, unsigned int ts) 
{
/*
 * Fichiers de sauvegardes:
 *
 * files/def_monde    |
 * files/def_entite   | monde par defaut
 *
 * files/parties.idx
 *   -> contient les timestamps commence / sauvegarde des parties
 *
 * files/monde_TIMESTAMP
 * files/entite_TIMESTAMP
 * files/backlog_TIMESTAMP
 *
 */
  char path[PATH_MAX];
  char nom[PATH_MAX];
  time_t myts;
  char *cwd;
  FILE *hand;
  int i;
  int n, ok = 0;
  struct stat tmp;

  if (!ts)
  {

    if (*monde)
    {
      libere_monde (*monde);
      *monde = NULL;
    }

    if (*entites)
    {
      libere_entites (*entites);
      *entites = NULL;
    }
  }

  if (check_rep())
  {
    fprintf (stderr, "Le rpertoire des sauvegardes n'existe pas.\n");
    return (-1);
  }

  cwd = my_getcwd ();

  if (!ts)
  {
    /* Sauvegarde NON automatique */

    if (!mode)
    {
      sprintf (path, "%s/%s/def_monde.eco", cwd, SAUVE_REP);
      *monde = charger_monde (path);
      if (*monde)
      {
        sprintf (path, "%s/%s/def_entites.eco", cwd, SAUVE_REP);
        *entites = charger_entites (path);
      }
    }
    else
    {
      sprintf (path, "%s/%s/parties.idx", cwd, SAUVE_REP); 
      hand = fopen (path, "r");

      i = 1;

      while (fscanf (hand, "%i - %s\n", (int*)&myts, nom) == 2)
      {
        fprintf(stdout, "Partie %i: %s\n", i, nom);
        i ++;
      }

      if (i == 1)
      {
        set_return_code (E_PASDEPARTIES);
        fprintf (stderr, "Pas de parties a charger.\n");
        return (0);
      }

      do
      {
        fprintf (stdout, "Donner le numero de partie a charger: ");
        fflush (stdout);

        if (fscanf (stdin, "%i", &n) == 1)
        {
          vide_buffer ();

          /* le nombre est la, voyons si il est ok */
          if (n <= 0 || n >= i)
          {
            set_return_code (E_BAD_USER);
            fprintf (stderr, "Numero de partie incorrect.\n");
            continue ;
          } 
          else
          {
            fseek (hand, 0, SEEK_SET);

            for (i = 0; i < n; i ++)
              fscanf (hand, "%i - %s\n", (int*)&myts, nom);

          fprintf (stdout, "Donc, chargement de %s ...\n", nom);
          ok = 1;
          }
        } 
        else
        {
          set_return_code (E_BAD_USER);
          fprintf (stderr, "Numero de saisie incorrect.\n");
        }
      }
      while (!ok);

      fclose (hand); 
    /* on a plus besoin de hand ... */

      sprintf (path, "%s/%s/%s_monde.eco", cwd, SAUVE_REP, nom);
      *monde = charger_monde (path);
      if (*monde)
      {
        sprintf (path, "%s/%s/%s_entites.eco", cwd, SAUVE_REP, nom);
        *entites = charger_entites (path);
      }
    }

    /* entites */
    if (*monde)
    {
      fprintf (stdout, "Monde charg !\n");
    }

    if (*entites)
    {
      fprintf (stdout, "Entites charges !\n");
    }
  
  }
  else
  {
    /* chargement d'une auvegarde automatique */
    sprintf (path, "%s/%s/%s/autosave_%i_monde.eco", cwd, SAUVE_REP, AUTOSAVE, ts);
    if (stat (path, &tmp) != 0)
    {
      my_free (cwd);
      return (-1);
    }
 
    fprintf(stdout, "chargement de %s\n", path);

    if (*monde)
    {
      libere_monde (*monde);
      *monde = NULL;
    }

    if (*entites)
    {
      libere_entites (*entites);
      *entites = NULL;
    }

    *monde = charger_monde (path);

    if (*monde)
    {
      sprintf (path, "%s/%s/%s/autosave_%i_entites.eco", cwd, SAUVE_REP, AUTOSAVE, ts);
      *entites = charger_entites (path);
    }
  }

  my_free (cwd);

  return (0);
}

int creer_jeu (struct monde_t **monde, cel **entites, cel *objets)
{
  /* Creer un jeu */
  /* Saisie par l'utilisateur de ce qu'il veut mettre. */
  /* monde et entites sont NULL */

  *monde = creer_monde ();
  if (!*monde)
    return (-1);

  fprintf (stdout, "Monde OK\n");

  *entites = creer_entites ((*monde)->taille_x, (*monde)->taille_y, objets);

  return (0);
}

/*
 * Sauvegarder un jeu.
 *
 */

int sauvegarder_jeu (struct monde_t *monde, cel *entites, unsigned int ts)
{
  cel *tmp_ent ;
  time_t timest;
  char nom[PATH_MAX];
  char file[PATH_MAX];
  char *path;
  FILE *hand;
  int i;

  timest = time (NULL);

  if (ts == 0)
  {
    do
    {
      /* Demander le nom du jeu */
      fprintf(stdout, "Donner le nom de la sauvegarde: ");
      fflush (stdout);
      while ((fscanf (stdin, "%s", nom) != 1));
      vide_buffer ();
      if (strlen (nom) <= 3)
       fprintf (stderr, "Nom de sauvegarder trop court.\n");
    } 
    while (strlen (nom) <= 3);
  }
  else
  {
    sprintf (nom, "autosave_%i", ts);
  } 

  /* Tester si l'on peut crire un nouveau fichier dans le repertoire */
  path = my_getcwd ();

  if (!ts)
    sprintf(file, "%s/%s/%s_monde.eco", path, SAUVE_REP, nom);
  else
    sprintf(file, "%s/%s/%s/%s_monde.eco", path, SAUVE_REP, AUTOSAVE, nom); 

  if (!ts)
    fprintf(stdout, "sauvegarde de %s\n", file);

  hand = fopen (file, "w");

  if (!hand)
  {
    fprintf(stderr, "Le jeu ne peut etre sauvegard.\nLe fichier %s est invalide.\n", file);
    set_return_code (E_INPUT_OUTPUT);
    my_free (path);
    return (-1);
  }

  /* Sauvegarde du monde */

  fprintf(hand, "%d %d %d %d %d %d\n", monde->taille_x, monde->taille_y, monde->cycle,
       monde->past, monde->present, monde->future);

  for (i = 0; i < monde->taille_x * monde->taille_y; i ++)
    fputc (monde->tabland[i], hand);

  fclose (hand);


  /* Generer les fichiers de data */

  if (!ts)
    sprintf (file, "%s/%s/%s_entites.eco", path, SAUVE_REP, nom);
  else
    sprintf (file, "%s/%s/%s/%s_entites.eco", path, SAUVE_REP, AUTOSAVE, nom);

  hand = fopen (file, "w");

  if (!hand)
  {
    fprintf(stderr, "Le jeu ne peut etre sauvegard.\nLe fichier %s est invalide.\n", file);
    set_return_code (E_INPUT_OUTPUT);
    my_free (path);
    return (-1);
  }

  tmp_ent = entites;

  while (tmp_ent)
  {
    fprintf (hand, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n", 
      ((struct entite_t*)CEL_CONT(tmp_ent))->espece,
      ((struct entite_t*)CEL_CONT(tmp_ent))->x,
      ((struct entite_t*)CEL_CONT(tmp_ent))->y,
      ((struct entite_t*)CEL_CONT(tmp_ent))->vie,
      ((struct entite_t*)CEL_CONT(tmp_ent))->pt_eat,
      ((struct entite_t*)CEL_CONT(tmp_ent))->pt_mov,
      ((struct entite_t*)CEL_CONT(tmp_ent))->pt_rep,
      ((struct entite_t*)CEL_CONT(tmp_ent))->pt_ter,
      ((struct entite_t*)CEL_CONT(tmp_ent))->pt_med,
      ((struct entite_t*)CEL_CONT(tmp_ent))->pt_arm,
      ((struct entite_t*)CEL_CONT(tmp_ent))->pt_mut,
      ((struct entite_t*)CEL_CONT(tmp_ent))->pt_sci,
      ((struct entite_t*)CEL_CONT(tmp_ent))->pt_bra,
      ((struct entite_t*)CEL_CONT(tmp_ent))->pt_utf,
      ((struct entite_t*)CEL_CONT(tmp_ent))->pt_pmp,
      ((struct entite_t*)CEL_CONT(tmp_ent))->pt_42
      );

    tmp_ent = CEL_NEXT(tmp_ent); 
  }

  fclose (hand);

  /* Ajouter dans le parties . idx */

  if (!ts)
  {
    sprintf (file, "%s/%s/parties.idx", path, SAUVE_REP);

    hand = fopen (file, "a+");

    if (!hand)
    {
      fprintf (stderr, "Ne peut pas achever la sauvegarde,"
                       " probleme dans le fichier d'index.\n");
      set_return_code (E_INPUT_OUTPUT);
      my_free (path);
      return (-1);
    }
    fprintf (hand, "%i - %s\n", (int)timest, nom);

    fclose (hand);
  }

  my_free (path);

  if (!ts)
    fprintf (stdout, "Sauvegarde effectuee avec succes ...\n");

  return (0);
}
